% compute texture regularity by ensemble composition, by dengxin dai

function f_irregularity = irregularity_run_fast_img(I)

    iterNum = 80;                           % the number of ensemble inpainting 

    F = makeSfilters; 
    f_num = size(F,3); 
    
    [H,W,Z] = size(I);
    if (Z==3) I = double(rgb2gray(I)); Z = 1; end

    I_out = zeros(H, W, f_num); 
    for i=1:f_num
       I_out(:,:, i)  = filter2(F(:,:, i),I);
    end
    I = I_out; 
    [H,W,Z] = size(I);

   
   % self-variance normalization 
%     I = double(I); 
%     I_mean = mean2(I);    % normalize the image via its standard variance 
%     I_std = std2(I); 
%     I = (I-repmat(I_mean, [H W 1])) ./ I_std .* 255;
       
    %% patch extraction 
    psize = 1; 
    pstep = 2; 
    [pf, ind_x, ind_y, num_x, num_y] = patchExtraction(double(I), int32(Z), int32(psize), int32(pstep)); 
    num_p = num_x*num_y;
    pf = reshape(pf, [psize.^2*Z num_p]);     % 
    pf = permute(pf, [2 1]);
    pf = double(pf); 
       
    %% obtaining the x-offset and y-offset of the texture (dominant offsets)
    offsetMat = zeros(2*H-1, 2*W-1);
    
    cluster_num = 200;                   % fast way for searching neighbors 
    nn_num = 60;   %number of knn  

    suppression_area = psize*2; % suppress the nearest 25 

    [ccenters, kids] = vl_kmeans(pf', cluster_num); 
    %cluster_num = unique(kids);

    for i_cind=1:cluster_num         % over all clusters, histogram of all shifts and add up       
        i_NN = find(kids==i_cind); 
        
        x_NN = ind_x(i_NN); 
        y_NN = ind_y(i_NN); 
        x_offsets = compute_shift(x_NN) + W;
        y_offsets = compute_shift(y_NN) + H; 
      
        offsets = (x_offsets-1)*(2*H-1) + y_offsets; 
        offsets = hist(offsets(:), 1:(2*H-1)*(2*W-1)); 
        offsets = reshape(offsets, [2*H-1, 2*W-1]); 
        offsetMat = offsetMat + offsets; 
    end
    
    offsetMat(H-suppression_area:H+suppression_area, W-suppression_area:W+suppression_area) = 0; %  suppress very spatially close patches 
    offsetMat(:, W:end) = 0; % symmetric values set to zero 
    offsetMat(H:end, :) = 0; 
    %%  matrix local maximum 
    label_num =  25;    % the number of dominant sift vectors  
    
    hLocalMax = vision.LocalMaximaFinder;
    hLocalMax.MaximumNumLocalMaxima = label_num;
    hLocalMax.NeighborhoodSize = [21 21];
    hLocalMax.Threshold = 1;

    location = step(hLocalMax, offsetMat);

    offsetY = double(location(:,2)) - H; 
    offsetX = double(location(:,1)) - W; 
 
        %% ensemble inpainting 
   I_stack = zeros(H, W, Z, label_num) + exp(10);   % void area with very large values to avoid copying pixel from void area 
   
   for il=1:label_num    
       ori_x1 = 1-offsetX(il); 
       ori_x2 = W; 
       ori_y1 = 1-offsetY(il); 
       ori_y2 = H;     
       tar_x1 = 1; 
       tar_x2 = 1+(ori_x2-ori_x1); 
       tar_y1 = 1; 
       tar_y2 = 1+(ori_y2-ori_y1); 
       
       I_stack(tar_y1:tar_y2, tar_x1:tar_x2, :, il) = I(ori_y1:ori_y2, ori_x1:ori_x2, :);    
   end

   % cording error sampling,       
    for iii=1:iterNum  
        rect_h = round(H/5); %randi([round(H/5), round(H*1/4)]);
        rect_w = round(W/5);     %randi([round(W/5), round(W*1/4)]);    %width & height 
        rect_x = randi(W-rect_w-1);                    % coords of top-left corner
        rect_y = randi(H-rect_h-1);

        dataCost = ones(label_num, rect_h, rect_w)*-1; 
        
        for il=1:label_num  
            dataCost(il, :,:) = sum(abs(I(rect_y:rect_y+rect_h-1,rect_x:rect_x+rect_w-1,:) - I_stack(rect_y:rect_y+rect_h-1,rect_x:rect_x+rect_w-1,:, il)), 3);  
        end
          
        dataCost = reshape(dataCost, [label_num, rect_h*rect_w]); 
        %% neighbors
        neighborW = zeros(rect_h*rect_w, rect_h*rect_w); 
        index = 1:rect_h*(rect_w-1); 
        index2 = index + rect_h; 
        index = (rect_w*rect_h)*(index2-1)+ index; 
        neighborW(index) = 1;  %  right neighbors 

        index = rect_h+1:rect_h*rect_w;
        index2 = index - rect_h; 
        index = (rect_w*rect_h)*(index2-1) + index; 
        neighborW(index) = 1;  %  left neighbors 

        ind_mat = ones(rect_h, rect_w); 
        ind_mat(1,:) = 0;
        index = find(ind_mat>0);
        index2 = index - 1; 
        index = (rect_w*rect_h)*(index2-1) + index; 
        neighborW(index) = 1;  %  top neighbors 

        ind_mat = ones(rect_h, rect_w); 
        ind_mat(end,:) = 0;
        index = find(ind_mat>0);
        index2 = index + 1; 
        index = (rect_w*rect_h)*(index2-1) + index; 
        neighborW(index) = 1;  %  bottom neighbors 

  %% smooth  
    lambda = 50;    %trandoff between data and smooth 
    smoothCost = ones(label_num).*lambda;
    smoothCost(1:label_num+1:end) = 0;
    label_cost = 1000; 
    
    h = GCO_Create(rect_h*rect_w,label_num);             % Create new object with NumSites=4, NumLabels=3
    GCO_SetDataCost(h, dataCost);   % Site  3   prefers label 3

    GCO_SetSmoothCost(h,(smoothCost));   % 
    GCO_SetLabelCost(h,label_cost);   % 
    GCO_SetNeighbors(h,neighborW);
    GCO_Expansion(h);                % Compute optimal labeling via alpha-expansion 

    a = GCO_GetLabeling(h);
    [E(iii) D S] = GCO_ComputeEnergy(h);   % Energy = Data Energy + Smooth Energy
    GCO_Delete(h);                % Delete the GCoptimization object when fin

    regularity(iii, 1) =  double(E(iii))./(rect_h*rect_w); %normalized by the size 
    regularity(iii, 2) =  double(D)./(rect_h*rect_w);
    regularity(iii, 3) =  double(S)./(rect_h*rect_w);
   
    end

    regularity_abs = mean(regularity, 1);
% %     regularity_rel = mean(regularity(:,2)./fill_energy');
% %     fill_energy = mean(fill_energy(:));
      
    f_irregularity = (regularity_abs(1)/255); 
end 

%% 
function [flag_valid, pixel] = copyPixel(img, j,i, offsetY, offsetX, l)
    
% if l==1    % itself
%    flag_valid = 1; 
%    pixel = img(j, i,:);
%    pixel = pixel(:); 
%    
% %     if isequal(pixel, [0 0 0]')        %copy from occlusion 
% %        flag_valid = 0; 
% %     end
% else 
    [imgH, imgW, Z] = size(img); 
    new_y = j + offsetY(l); 
    new_x = i + offsetX(l);
    
    flag_valid = 0; 
    pixel = [0 0 0]; 
    
    if new_x > 0 && new_x <= imgW && new_y > 0 && new_y <= imgH
       pixel = img(new_y, new_x,:); 
       pixel = squeeze(pixel); 
       flag_valid = 1; 
%        if isequal(pixel, [0 0 0]')        %copy from occlusion 
%            flag_valid = 0; 
%        end
    end
end

%% 
% x should be a vector [N, 1], diff to itself [N, N]
function diff_matrix = compute_shift(x)
    x = x(:);  

    [N] = length(x); 

    x_mat1 = repmat(x, [1, N]);
    x_mat2 = repmat(x', [N, 1]); 

    diff_matrix = x_mat1 - x_mat2; 
end



