#include "mex.h"
#include <string.h>
// output: the rgb feature of all image patches, x positions, y positions of left_top pixel of the patch, x_nums, y_nums 
//(the number of y positions)
// input:  image, patch size, patch step 

/* Always include this */
void mexFunction(int nlhs, mxArray *plhs[],
/* Output variables */
int nrhs, const mxArray *prhs[]) /* Input variables */
{

    // veryfy inputs, img, patch_size, step 
    // outputs: f_rgb (vectered rgb of a patch), f_x, f_y 
   
    
    #define IMAGE prhs[0]
    #define Z     prhs[1] 
    #define PATCH_SIZE prhs[2]
    #define STEP prhs[3]

    double *img; 
    int *z; 
    int *patch_size; 
    int *step; 
         
    img           = (double*)  mxGetPr(IMAGE);   // image 
    z             = (int*)  mxGetPr(Z);
    patch_size    = (int*)  mxGetPr(PATCH_SIZE); 
    step          = (int*)  mxGetPr(STEP); 
     
    int img_h, img_w, img_z, psize, pstep;  
  
    img_h = mxGetM(IMAGE);
    img_w = mxGetN(IMAGE);
    img_z = z[0];
    psize = patch_size[0];
    pstep = step[0]; 
    
    img_w = img_w/img_z;   //rgb image or multiple bands 
    
        
    // positions 
    int x_nums = (img_w-psize+1)/pstep;    // the number of center pix in x dimension
    int y_nums = (img_h-psize+1)/pstep;    // the number of center pix in x dimension
    
    mwSize of_dims, ox_dims, xnums_dim;                  // dims of output vector 
    of_dims = x_nums*y_nums*psize*psize*img_z; 
    ox_dims = x_nums*y_nums; 
    xnums_dim = 1;
    
    
    //mexPrintf("%d ddd %d \n", x_nums, y_nums);
   
    
    //output 
    plhs[0] = mxCreateNumericArray(1, &of_dims, mxDOUBLE_CLASS, mxREAL);    //f_rgb
    plhs[1] = mxCreateNumericArray(1, &ox_dims, mxINT32_CLASS, mxREAL);    // f_x
    plhs[2] = mxCreateNumericArray(1, &ox_dims, mxINT32_CLASS, mxREAL);    // f_y
    plhs[3] = mxCreateNumericArray(1, &xnums_dim, mxINT32_CLASS, mxREAL);  // x_nums
    plhs[4] = mxCreateNumericArray(1, &xnums_dim, mxINT32_CLASS, mxREAL);  // y_nums


    double *of = (double*)mxGetData(plhs[0]);
    int *ox = (int*)mxGetData(plhs[1]);
    int *oy = (int*)mxGetData(plhs[2]);
    int *numx = (int*)mxGetData(plhs[3]);
    int *numy = (int*)mxGetData(plhs[4]);
    
    numx[0] = x_nums; 
    numy[0] = y_nums; 
    
    int in_pos = 0;   //current position in input image
    int out_pos_f = 0;  // current position in output image 
    int out_pos_x = 0; 
    
    for(int i=0; i<x_nums; i++)
    {
        for(int j=0; j<y_nums; j++)
        {
            ox[out_pos_x] = i*pstep+1;  // +1 for matlab index 
            oy[out_pos_x] = j*pstep+1; 
            out_pos_x++; 
            
            for (int zz=0; zz<img_z; zz++)      // rgb 
            {
                for(int k=0; k<psize; k++)      // x direction
                {

                        in_pos = zz*img_h*img_w + (i*pstep)*img_h + j*pstep + k*img_h; 

                          for(int o=0; o<psize; o++)  // y direction
                            {
                                of[out_pos_f+o] = img[in_pos+o]; 
                            }

                        //memcpy((void*)of[out_pos_f], (void*)img[in_pos], psize*sizeof(int));                    
                        out_pos_f += psize;         
                }
            }
            
        }
        
    }       
}


// mex patchExtraction.cpp

