%  evaluate synthesizability on ETH Synthesizability dataset
%  by Dengxin Dai (dai@vision.ee.ethz.ch) at ETH Zurich
%  Oct. 2013

clear all, clc
addpath(genpath('./libs/')); 
addpath(genpath('./features/')); 
addpath(genpath('./models/')); 

fet_name = {'textureness', 'homogeneity', 'repetitiveness', 'irregularity', ...
    'Sfilter', 'lbp', 'gist'};

%% load features and labels 
img_path = './ETH_Synthesizability/texture/'; 
fet_path =  './ETH_Synthesizability/feature/'; 
load('./ETH_Synthesizability/annotation/synthesizability_labeling.mat');
imgNum = length(TEXTURE);

is_load = 0;      % load all features and save it into one file
if is_load 
    for i=1:imgNum
        img_lab(i,1) = TEXTURE(i).quality;
    end
    
    img_fet  = [];                             %all img names
    for i=1:imgNum
        img_name = TEXTURE(i).name;
        fet_jj = []; 
        for jj=1:length(fet_set)
            fet_out = feature_loader(fet_path, img_name, fet_set{jj}); 
            fet_jj = cat(1, fet_jj, fet_out);
        end
        img_fet = cat(1, img_fet, fet_jj');     
    end
    save('./models/train_label_fet.mat', 'img_lab', 'img_fet');
else 
    load('./models/train_label_fet.mat'); 
end
%% training and test 
train_rate = 0.3;   % 30% training images
iter_num = 5;       % over 5 random training_test splits 
predict_acc = []; 
% img_fet = img_fet(:, 4:end); 

RR = [ 1.0 ];       % different levels of recall, e.g. [0.1 0.5 1.0] 

 for it = 1:iter_num
    img_index = randperm(imgNum); 

    train_ind = img_index(1:imgNum*train_rate);
    test_ind  = img_index(imgNum*train_rate+1:end);

    test_lab = img_lab(test_ind); 

    nTrees = 30;
    % Train the TreeBagger (Decision Forest).
    RF = TreeBagger(nTrees,img_fet(train_ind, :),img_lab(train_ind), 'Method', 'regression');
    [p_value, p_prob] = predict(RF, img_fet(test_ind, :));

     % average precision 
     retrieved_label_set = [0.5 1]; 
    for label_id=1:length(retrieved_label_set)   % for acceptable and good    
        for r_id=1:length(RR)                    % over all recalls
           predict_acc(label_id, r_id, it) = MapCompt(test_lab>=retrieved_label_set(label_id), p_value, RR(r_id)); 
        end
    end
   % precision finished  
 end

av_all = mean(predict_acc, 3)  % first row >= acceptable, second row >= good

disp('thanks'); 
