%  compute synthesizability of all images in the folder ./images/
%  by Dengxin Dai (dai@vision.ee.ethz.ch) at ETH Zurich
%  Oct. 2013

clear all, clc
addpath(genpath('./libs/')); 
addpath(genpath('./features/')); 
addpath(genpath('./models/')); 

fet_set = {'textureness', 'homogeneity', 'repetitiveness', 'irregularity', ...
    'Sfilter', 'lbp', 'gist'}; 

removing_duplicate = 0; % 1 to remove the test image from ETH Synthesizability, 
                        % if test images are not from ETH Synthesizability,
                        % use 0 in order to train the model using all data
                        % 0 train a model for all test images, 1 train a
                        % model for each test image
%% load features and labels 
img_path = './ETH_Synthesizability/texture/'; 
fet_path =  './ETH_Synthesizability/feature/'; 
load('./ETH_Synthesizability/annotation/synthesizability_labeling.mat');
imgNum = length(TEXTURE);

is_load = 0;      % load all features and save it into one file
if is_load 
    for i=1:imgNum
        img_lab(i,1) = TEXTURE(i).quality;
    end
    
    img_fet  = [];                             %all img names
    for i=1:imgNum
        img_name = TEXTURE(i).name;
        fet_jj = []; 
        for jj=1:length(fet_set)
            fet_out = feature_loader(fet_path, img_name, fet_set{jj}); 
            fet_jj = cat(1, fet_jj, fet_out);
        end
        img_fet = cat(1, img_fet, fet_jj');     
    end
    save('./models/train_label_fet.mat', 'img_lab', 'img_fet');
else 
    load('./models/train_label_fet.mat'); 
end

% test sub_set of features 
dim_subset = 1:96;  % dims of the 7 fets are 1, 1, 1, 1, 13, 59, 20, resp. 
img_fet = img_fet(:, dim_subset);

%% training the model
nTrees = 30;
if ~removing_duplicate
    train_fet = img_fet; 
    train_lab = img_lab; 
    RF = TreeBagger(nTrees,train_fet,train_lab, 'Method', 'regression'); 
end
%% test, for all imgs in one folder 
vl_setup; 
norm_size = [300 300];               % resize test images to the same size as training ones  
img_folder = './images/'; 
img_set = dir(strcat(img_folder, '*.jpg')); 

for ii=1:length(img_set)    

      img = imread(strcat(img_folder, img_set(ii).name)); 
      img = imresize(img, norm_size); 
      %computing feature
      test_fet = []; 
      for i=1:length(fet_set)             % over all features   
       fname = fet_set{i}; 
           fet = feature_compute(img, fname);
           test_fet = cat(2, test_fet, fet(:)'); 
      end   
   
    %%  training with reduced data
    if removing_duplicate % removing the img from training data 
      [xx, same_ind] = knn(test_fet', img_fet', 1);   
      train_fet = img_fet; 
      train_fet(same_ind,:)  = []; 
      train_lab = img_lab; 
      train_lab(same_ind,:) = []; 
      RF = TreeBagger(nTrees,train_fet,train_lab, 'Method', 'regression'); 
    end
      
     %% predict
     [syn_score(ii), p_prob] = predict(RF, test_fet);
     fprintf('the %d th image processed ... \n', ii);
end
%% display imgs (the first 30) with their synthesizability 
[f_value, f_ind] = sort(syn_score, 'descend');
figure;
for n=1:min(30, length(img_set))
    subplot(5, 6, n); % figure(n);
    imshow(imresize(imread(strcat(img_folder, img_set(f_ind(n)).name)), norm_size));
        text(0.5, 0.5, num2str(f_value(n)));
end

disp('thanks');



